@@ -1,5 +1,5 @@
 #' Download an elevation raster from internet
-#' 
+#'
 #' The function `getelev` downloads an elevation raster from internet. It
 #' is a wrapper that 1) calls the function [elevatr::get_elev_raster] to
 #' download the data and 2) saves the downloaded raster on the hard drive (so
@@ -14,21 +14,21 @@
 #' increasing the value of the argument `z`. You can also restrict the area
 #' to be downloaded using the arguments `long_min`, `long_max`, `lat_min` &
 #' `lat_max`.
-#' 
+#'
 #' Note that when using [prepraster] you will be able to reduce the resolution
 #' and restrict the boundaries of this elevation raster, but you won't be able
 #' to increase the resolution or expend the boundaries. As a consequence, it is
 #' probably a good idea to overshoot a little when using `getelev` and
 #' download data at a resolution slightly higher than you need and for a extent
 #' larger than your data.
-#' 
+#'
 #' You can customise further what you download by using other parameters of
 #' [elevatr::get_elev_raster] (via the elipsis `...`).
-#' 
+#'
 #' Please refer to the documentation of
 #' [elevatr::get_elev_raster] for information on the sources and follows link in
 #' there to know how to cite them.
-#' 
+#'
 #' @inheritParams prepsources
 #' @param file A *string* indicating where to store the file on the hard
 #'   drive (default = `~/elevation_world_z5.tif`)
@@ -51,11 +51,11 @@
 #'
 #' @return This function returns the full path where the file has been stored
 #' @examples
-#' 
+#'
 #' ## To download the high resolution
 #' ## raster at the default location, just type:
 #' ## getelev()
-#' 
+#'
 #' @export
 getelev <- function(file = "~/elevation_world_z5.tif",
                     z = 5,
@@ -68,20 +68,18 @@ getelev <- function(file = "~/elevation_world_z5.tif",
                     overwrite = FALSE,
                     Ncpu = getOption_IsoriX("Ncpu"),
                     verbose = interactive(),
-                    ...
-) {
-  
+                    ...) {
   ## Checking that elevatr is installed
   if (!requireNamespace("elevatr", quietly = TRUE)) {
     stop("You must install the package elevatr for this function to run: `install.packages('elevatr')`")
   }
-  
+
   ## Turning path into canonical form
   ## (this avoids the problem of using the wrong slashes and so on)
   file <- normalizePath(file, mustWork = FALSE)
-  
+
   path <- dirname(file)
-  
+
   if (path == ".") {
     path <- getwd()
   } else if (!dir.exists(path)) {
@@ -91,13 +89,12 @@ getelev <- function(file = "~/elevation_world_z5.tif",
     }
     dir.create(path, recursive = TRUE)
   }
-  
+
   ## Applying margin_pct
   if (margin_pct != 0) {
-    
-    margin_long_extra <- (long_max - long_min) * margin_pct/100
-    margin_lat_extra  <- (lat_max - lat_min) * margin_pct/100
-    
+    margin_long_extra <- (long_max - long_min) * margin_pct / 100
+    margin_lat_extra <- (lat_max - lat_min) * margin_pct / 100
+
     if (long_min > -180) {
       long_min <- max(c(-180, long_min - margin_long_extra))
     }
@@ -111,40 +108,43 @@ getelev <- function(file = "~/elevation_world_z5.tif",
       lat_max <- min(c(90, lat_max + margin_lat_extra))
     }
   }
-  
+
   ## Conditional file download
-  if (file.exists(file) & !overwrite) {
-    message(paste("the file", basename(file), "is already present in", path,
-                  "so it won't be downloaded again unless you set the argument overwrite to TRUE\n"
-    )
-    )
+  if (file.exists(file) && !overwrite) {
+    message(paste(
+      "the file", basename(file), "is already present in", path,
+      "so it won't be downloaded again unless you set the argument overwrite to TRUE\n"
+    ))
   } else {
-    
     if (verbose) print("Downloading and formating the elevation raster... (be patient)")
-    elev <- elevatr::get_elev_raster(locations = data.frame(x = c(long_min, long_max),
-                                                            y = c(lat_min, lat_max)),
-                                     z = z,
-                                     prj = "+proj=longlat +datum=WGS84 +no_defs",
-                                     clip = "bbox",
-                                     override_size_check = override_size_check,
-                                     ncpu = Ncpu,
-                                     verbose = verbose,
-                                     ...)
-    
+    elev <- elevatr::get_elev_raster(
+      locations = data.frame(
+        x = c(long_min, long_max),
+        y = c(lat_min, lat_max)
+      ),
+      z = z,
+      prj = "+proj=longlat +datum=WGS84 +no_defs",
+      clip = "bbox",
+      override_size_check = override_size_check,
+      ncpu = Ncpu,
+      verbose = verbose,
+      ...
+    )
+
     if (verbose) print("Writing the elevation raster on the disk...")
     terra::writeRaster(elev, filename = file, overwrite = overwrite)
     if (verbose) print("Done.")
   }
-  
+
   message("you can load your elevation raster as follows:")
   message(paste0("elev_raster <- terra::rast('", file, "')"))
-  
+
   return(invisible(file))
 }
 
 
 #' Download rasters of monthly precipitation from internet
-#' 
+#'
 #' The function `getprecip` allows for the download of rasters of monthly
 #' precipitation from internet. It downloads the "precipitation (mm) WorldClim
 #' Version 2.1" at a spatial resolution of 30 seconds (~1 km2). After download,
@@ -154,13 +154,13 @@ getelev <- function(file = "~/elevation_world_z5.tif",
 #' [prepcipitate]. It can then be used to predict annual averages
 #' precipitation weighted isoscapes with the function
 #' [isomultiscape].
-#' 
-#' In the argument "path" is not provided, the file will be stored in the 
-#' current working directory. The functions can create new directories, so you 
+#'
+#' In the argument "path" is not provided, the file will be stored in the
+#' current working directory. The functions can create new directories, so you
 #' can also indicate a new path. The integrity of the elevation raster is tested
 #' after a call to `getprecip`. In case of corruption, try downloading the
 #' file again, specifying overwrite = TRUE to overwrite the corrupted file.
-#' 
+#'
 #' @inheritParams getelev
 #' @param path A *string* indicating where to store the file on the hard
 #'   drive (without the file name!). Default = current directory.
@@ -170,24 +170,22 @@ getelev <- function(file = "~/elevation_world_z5.tif",
 #'   session and `FALSE` otherwise. If a *numeric* is provided instead,
 #'   additional information about the download will be provided if the number is
 #'   greater than 1.
-#'   
+#'
 #' @return This function returns the path of the folder where the files have
 #'   been stored
 #'
 #' @source \url{https://worldclim.org/data/worldclim21.html}
 #' @examples
-#' 
+#'
 #' ## To download the monthly precipitation
 #' ## in your current working
 #' ## directory, just type:
-#' ## getprecip()
+#' ## getprecip(path = "~/Downloads/")
 #' ## Mind that the file weights ca. 1GB!
 #' @export
 getprecip <- function(path = NULL,
                       overwrite = FALSE,
-                      verbose = interactive()
-) {
-  
+                      verbose = interactive()) {
   ## check options
   old_opts <- options()
   if (options()$timeout == 60) {
@@ -197,44 +195,45 @@ getprecip <- function(path = NULL,
     message("You are using a custom value of timeout lower than 600s. This could be unsufficient for downloading the large file. If the download crashes as a result, please call `options(timeout = XX)` with XX the number of seconds you want to allow R waiting for the download to be completed before crashing. After this, rerun `getprecip()`.")
   }
   on.exit(options(old_opts))
-  
+
   ## Use current directory if path is missing
   if (is.null(path)) {
     path <- getwd()
   }
-  
+
   ## Normalise path the remove last slash
   path <- base::normalizePath(path, mustWork = FALSE)
-  
+
   ## Define web address and file name
   address_precip <- "https://biogeo.ucdavis.edu/data/worldclim/v2.1/base/wc2.1_30s_prec.zip"
   filename_precip <- "wc2.1_30s_prec.zip"
-  
+
   ## Define md5sum
   ## (created with tools::md5sum("wc2.1_30s_prec.zip"))
   md5sum_precip <- "cc100350d034883c9e925c903fb3c7c3"
-  
+
   ## Download and check file
-  path_to_zip <- downloadfile(address = address_precip,
-                              filename = filename_precip,
-                              path = path,
-                              overwrite = overwrite,
-                              md5sum = md5sum_precip,
-                              verbose = verbose
+  path_to_zip <- downloadfile(
+    address = address_precip,
+    filename = filename_precip,
+    path = path,
+    overwrite = overwrite,
+    md5sum = md5sum_precip,
+    verbose = verbose
   )
-  
+
   ## Unzip the file
   if (verbose > 0) {
     print("unzipping in progress...", quote = FALSE)
   }
   outpath <- paste0(path, "/wc2.1_30s_prec")
   utils::unzip(path_to_zip, exdir = outpath)
-  
+
   if (verbose > 0) {
     print("unzipping done!", quote = FALSE)
     print(paste("The files can be found in the folder", outpath), quote = FALSE)
   }
-  
+
   return(invisible(outpath))
 }
 
@@ -270,29 +269,27 @@ getprecip <- function(path = NULL,
 #' @export
 #'
 #' @seealso [getelev()], [getprecip()]
-#' 
-#' 
+#'
+#'
 downloadfile <- function(address = NULL, filename = NULL, path = NULL,
-                         overwrite = FALSE, md5sum = NULL, verbose = interactive()
-) {
-  
+                         overwrite = FALSE, md5sum = NULL, verbose = interactive()) {
   if (verbose > 0) {
     print(paste("the function attempts to download", filename, "from internet"), quote = FALSE)
   }
-  
+
   ## Change internet options to display more information
   opt_ori <- options()$internet.info
   if (verbose > 1) options(internet.info = 1)
-  
+
   ## Use current directory if path is missing
   if (is.null(path)) {
     path <- getwd()
   }
-  
+
   ## Turning path into canonical form
   ## (this avoids the problem of having terminal slash or not)
   path <- normalizePath(path, mustWork = FALSE)
-  
+
   ## Create directory if missing
   if (!dir.exists(path)) {
     if (verbose > 0) {
@@ -300,19 +297,19 @@ downloadfile <- function(address = NULL, filename = NULL, path = NULL,
     }
     dir.create(path, recursive = TRUE)
   }
-  
+
   ## Conditional file download
   complete_path <- paste(path, filename, sep = "/")
-  if (file.exists(complete_path) & !overwrite) {
-    message(paste("the file", filename, "is already present in", path,
-                  "so it won't be downloaded again unless you set the argument overwrite to TRUE"
-    )
-    )
+  if (file.exists(complete_path) && !overwrite) {
+    message(paste(
+      "the file", filename, "is already present in", path,
+      "so it won't be downloaded again unless you set the argument overwrite to TRUE"
+    ))
   } else {
     time <- system.time(utils::download.file(address, destfile = complete_path, mode = "wb"))
     message(paste0("The download lasted ", time[["elapsed"]], "s"))
   }
-  
+
   ## Checking MD5sum
   if (!is.null(md5sum)) {
     if (tools::md5sum(complete_path) == md5sum) {
@@ -321,15 +318,14 @@ downloadfile <- function(address = NULL, filename = NULL, path = NULL,
       warning("the file seems to be corructed (md5sums do not match). Try to download it again setting the argument overwrite to TRUE.")
     }
   }
-  
+
   ## Display outcome
   if (verbose > 0) {
     print(paste("the file", filename, "is stored in the folder", path), quote = FALSE)
   }
-  
+
   ## Restore original internet options
   options(internet.info = opt_ori)
-  
+
   return(invisible(complete_path))
 }
-
