@@ -89,4 +89,15 @@ terra::values
 
 #' @importFrom terra cellSize
 #' @export
-terra::cellSize
\ No newline at end of file
+terra::cellSize
+
+
+# Imports not reexported --------------------------------------------------
+
+#' @importFrom terra saveRDS
+#' @export
+NULL # to not shadow IsoriX doc
+
+#' @importFrom terra readRDS
+#' @export
+NULL # to not shadow IsoriX doc
